/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * main.c - demo main module to test irmp decoder
 *
 * Copyright (c) 2009-2010 Frank Meyer - frank(at)fli4l.de
 *
 * $Id: main.c,v 1.7 2010/06/22 08:33:21 fm Exp $
 *
 * ATMEGA88 @ 8 MHz
 *
 * Fuses: lfuse: 0xE2 hfuse: 0xDC efuse: 0xF9
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * uncomment this for codevision compiler:
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
// gcc compiler
#include <inttypes.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <string.h>
#include "irmp.h"
#include "irmpconfig.h"
#include "lcd3.h"
#include "bits.h"
#include "uart.h"
#include "irsnd.h"
#include "ir_tools.h"

static char *Proto[]={ \
"SIRCS","NEC","SAMSUNG","MATSUSH","KASEIKYO","RECS80","RC5(x)","DENON","RC6","SAMSG32",\
"APPLE","RECS80x","NUBERT","BANG_OLUF","GRUNDIG","NOKIA","SIEMENS","FDC","RCCAR","JVC"};

char s[7], gg;
uint16_t count__;
uint16_t count2__;

#ifndef F_CPU
#error F_CPU unkown
#endif

#define IR_FREQUENZ 36000L

void timer_init(void) // timer 1
{
// gcc
  OCR1A   =  (F_CPU / F_INTERRUPTS) - 1;                                    // compare value: 1/10000 of CPU frequency
  TCCR1B  = (1 << WGM12) | (1 << CS10);                                     // switch CTC Mode on, set prescaler to 1

#if defined (__AVR_ATmega8__) || defined (__AVR_ATmega16__) || defined (__AVR_ATmega32__) || defined (__AVR_ATmega64__) || defined (__AVR_ATmega162__) 
  TIMSK  |= 1 << OCIE1A;                                                     // OCIE1A: Interrupt by timer compare
#else
  TIMSK1  |= 1 << OCIE1A;                                                    // OCIE1A: Interrupt by timer compare
#endif  // __AVR...
}

void timer0_init(void)
{
  	OCR0A	= (F_CPU / ( 2 * IR_FREQUENZ )) - 1;               // compare value: 
#if !FB_LCD_STECKBRETT
	IR_SND_DDR |= (1<<IR_SND);                                  // set DDR to output
  	IR_SND_PORT &= ~(1<<IR_SND);                                // set pin to low
#endif
}

void timer0_start(void)
{
  	TCCR0A  |= (1 << WGM01) | (1 << WGM00) | (1 << COM0A0);   // switch CTC Mode on, set prescaler to 1
	TCCR0B 	|= (1 << WGM02) | (1<<CS00);
  	TIMSK0  |= (1 << OCIE0A);                                   // OCIE0A: Interrupt by timer compare
}

void timer0_stop(void)
{
	TIMSK0  &= ~(1 << OCIE0A);                                   // OCIE0A: Interrupt by timer compare
	TCCR0B 	&= ~(1 << WGM02) | (1<<CS00);
  	TCCR0A  &= ~(1 << WGM01) | (1 << WGM00) | (1 << COM0A0);									

#if !FB_LCD_STECKBRETT
  	IR_SND_PORT &= ~(1<<IR_SND);                                // set out to low
#endif
}
/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * timer 1 compare handler, called every 1/10000 sec
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
// Timer 1 output compare A interrupt service routine
// gcc
ISR(TIMER1_COMPA_vect)
{
	(void) irmp_ISR();                                                        // call irmp ISR
	// call other timer interrupt routines...
	extern uint16_t count__;
	count__++;

	extern uint16_t count2__;
	count2__++;
}

ISR(TIMER0_COMPA_vect)
{
#if !FB_LCD_STECKBRETT
	IR_SND_PORT ^= (1<<IR_SND);    // XOR, Kurzschreibweise, PORTB = PORTB ^ (1<<PB0) 
#else
	cli();
	sei();
#endif
  // call other timer interrupt routines...
}

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * MAIN: main routine
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */

// This is the main routine if you use GCC Compiler
extern void irmp_uart_putc(unsigned char);

int main (void)
{
  	IRMP_DATA irmp_data;
	
  	uint8_t blinki=0, sek10=0, _blink=5, n, t0_lauft=0, alt_sek10=0, t2=0;
  	char temp[20];
  	char temp2[255];
  	char temp2_bak[255];
  	char _string[20];
	
	uint8_t _play=0;

  t0_lauft=0;

  IR_ERKANNT_DDR |= (1<<IR_ERKANNT);                                  // set pin to output
  IR_ERKANNT_PORT &= ~(1<<IR_ERKANNT);                                // set pin to low

  xRC5_VCC_DDR |= (1<<xRC5_VCC);                                  // set pin to output
  xRC5_VCC_PORT |= (1<<xRC5_VCC);                                // set pin to high

  DCF_LED_DDR |= (1<<_dcfLED);                                  // set pin to output
  DCF_LED_PORT &= ~(1<<_dcfLED);                                  // set pin to high
  DCF_in_DDR |= (1<<_DCF_in);                                  // set pin to output
  DCF_in_PORT &= ~(1<<_DCF_in);                                // set pin to low

  SPEED_DDR |= (1<<_speedLED);                                  // set pin to output
  SPEED_LED_PORT &= ~(1<<_speedLED);                                // set pin to low

#if IRMP_LOGGING != 0       // 1: log IR signal (scan), 0: do not (default)
    uart_init(UART_BAUD_SELECT(9600,F_CPU));
#endif

  irmp_init();                                                              // initialize rc5
  timer_init();                                                             // initialize timer
  timer0_init();                                                             // initialize timer

  sei ();                                                                   // enable interrupts
  /* initialize display, cursor off */
  lcd_init(LCD_DISP_ON);  // LCD_DISP_ON_CURSOR_BLINK
  
  // load two userdefined characters from program memory
  // into LCD controller CG RAM location 0 and 1
  lcd_command(_BV(LCD_CGRAM));  // set CG RAM start address 0 
  lcd_clrscr();
  lcd_gotoxy(0,0);

#if FB_LCD_STECKBRETT
  lcd_puts("LED Test Steckbrett ");

#else
  lcd_puts("LED Test jar timer  ");
  _blink=2;

#endif

  lcd_gotoxy(0,1);
  lcd_puts("grn   ");
  
  for(n=_blink;n>0;n--)
  	gruen_blink1s();

#if !FB_LCD_STECKBRETT

  lcd_gotoxy(0,1);
  lcd_puts("gelb   ");
  for(n=_blink;n>0;n--)
  	gelb_blink1s();


  lcd_gotoxy(0,1);
  lcd_puts("hell   ");
  for(n=_blink;n>0;n--)
  	speed_blink1s();


#endif

  lcd_clrscr();
  lcd_gotoxy(0,3);
  lcd_puts("ukw,jar IRMP 179");

  count__=0;
  count2__=0;
  sek10=0;
  strcpy(temp2_bak, "#");
  strcpy(temp2, "#");
  strcpy(_string, "");

	lcd_gotoxy(0,2);
  	strcpy(temp, "Logging Mode : "); strcat(temp, itoa(IRMP_LOGGING,s,10));

#if IRMP_LOGGING != 0       // 1: log IR signal (scan), 0: do not (default)
  	uart_puts( temp );
#endif

	lcd_puts(temp);

	for (;;)
  	{
/*
	if (t0_lauft==0 && count2__>=5) // 500s
	{
		timer0_start();
		t0_lauft=1;
	}

	if (t0_lauft==1 && count2__>=10) // 500s
	{
		timer0_stop();
		count2__=0;
		t0_lauft=0;
	}
*/
	irmp_data.protocol = IRMP_KASEIKYO_PROTOCOL;	irmp_data.address  = 0x2002;	irmp_data.flags    = 0;

	if (_play==0 && t2>9)
	{
		irmp_data.command  = 0xb0a0;		// play
		//irmp_data.command  = 0xb060;	// pause
		irsnd_send_data (&irmp_data, TRUE);
		_play=1;
		lcd_gotoxy(0,2);
  		lcd_puts("p l a y             ");
		t2=0;
		lcd_gotoxy(10,2);
  		lcd_puts(itoa(t2+1, s, 10));lcd_puts(" ");
	}
	if (_play==1 && t2>4)
	{
		//irmp_data.command  = 0xb0a0;		// play
		irmp_data.command  = 0xb060;	// pause
		irsnd_send_data (&irmp_data, TRUE);
		_play=0;
		lcd_gotoxy(0,2);
  		lcd_puts("P A U S E           ");
		t2=0;
		lcd_gotoxy(10,2);
  		lcd_puts(itoa(t2+1, s, 10));lcd_puts(" ");
		t2=0;
	}
	if(sek10!=alt_sek10)
	{
		lcd_gotoxy(10,2);
  		lcd_puts(itoa(t2+1, s, 10));lcd_puts(" ");
		t2++;
		alt_sek10=sek10;
	}

	if (count__>=F_INTERRUPTS) //10000
	{
		count__=0;
		sek10++;
		if(blinki)
		{	IR_ERKANNT_PORT &= ~(1<<IR_ERKANNT); blinki=0; 
			lcd_gotoxy(0,2);lcd_puts("                    ");
		}               // set pin to low
	}

    if (irmp_get_data (&irmp_data))
    {
		if(!strcmp((char *)Proto[irmp_data.protocol-1],"KASEIKYO"))
			gelb_blink1s();
		else
			gruen_blink1s();

		lcd_gotoxy(0,0); lcd_puts("R: Code:            ");
		lcd_gotoxy(9,0); lcd_puts((char *)Proto[irmp_data.protocol-1]);
		lcd_gotoxy(0,1); lcd_puts("A:       C:         ");
		lcd_gotoxy(3,1); lcd_puts(itoa(irmp_data.address,s,16)); 
		lcd_gotoxy(14,1); lcd_puts(itoa(irmp_data.command,s,16)); 
		lcd_gotoxy(0,2); lcd_puts("                    ");
  		
		if(!strcmp((char *)Proto[irmp_data.protocol-1], "RC5(x)"))
		{	lcd_gotoxy(5,2); 
			lcd_puts("("); lcd_puts(itoa(irmp_data.address,s,10)); lcd_puts(")");
		}
		
		lcd_gotoxy(14,1);	
		lcd_puts(itoa(irmp_data.command,s,16)); 

		if(!strcmp((char *)Proto[irmp_data.protocol-1], "RC5(x)"))
		{	lcd_gotoxy(14,2);	
			lcd_puts("("); lcd_puts(itoa(((irmp_data.address*100)+irmp_data.command),s,10)); lcd_puts(")"); }
        
		// ir signal decoded, do something here...
        // irmp_data.protocol is the protocol, see irmp.h
        // irmp_data.address is the address/manufacturer code of ir sender
        // irmp_data.command is the command code

		strcpy(temp2,"# Code: "); strcat(temp2,(char *)Proto[irmp_data.protocol-1]);
		strcat(temp2,", A: ");	strcat(temp2,itoa(irmp_data.address,s,16)); 
		strcat(temp2,", C: ");	strcat(temp2,itoa(irmp_data.command,s,16)); 
		strcat(temp2,"\r\n");

		/*

#if IRMP_LOGGING != 0       // 1: log IR signal (scan), 0: do not (default)
		if(strlen(temp2_bak)==0)
			uart_puts(temp2);
#endif
		*/

#if IRMP_LOGGING != 0       // 1: log IR signal (scan), 0: do not (default)
		if (strcmp(temp2, temp2_bak)!=0)
		{	strcpy(temp2_bak, temp2);
			uart_puts(temp2);
		}
#endif

    }

	if (sek10==25)
	{	lcd_gotoxy(0,0); lcd_puts("R: Code:            ");
		lcd_gotoxy(0,1); lcd_puts("A:       C:         ");
/*		lcd_gotoxy(0,2); 
		lcd_gotoxy(0,2); lcd_puts("                    ");
*/		sek10=0;
	}

  }
}





